;
;   Assembly language subroutines to monitor INT 21 for writes to STDOUT
;   or STDERR and redirect to both file and screen.
;
;		Bob Withers, July 1989
;

MsDos		equ	    21h
DosGetVector	equ	    35h
DosSetVector	equ	    25h
DosWriteFile	equ	    40h

BiosVideo	equ	    10h
BiosWriteTTY	equ	    0eh

STDOUT		equ	    1
STDERR		equ	    2

WPTR		equ	    word ptr

		.model	    small
		.code

		public	    _Grab21, _GiveUp21

OldInt21	dd	    ?

;   Save the current INT 21 vector and set it to point to our INT 21
;   pre-processor code.
;
_Grab21 	proc
		push	    ds			    ;Save regs (MSC conv)
		push	    si			    ;		.
		push	    di			    ;		.
		mov	    al,MsDos		    ;Save current INT 21
		mov	    ah,DosGetVector	    ;	 vector address
		int	    MsDos		    ;		.
		mov	    WPTR CS:OldInt21,bx     ;		.
		mov	    WPTR CS:OldInt21+2,es   ;		.
		mov	    ax,cs		    ;Install my INT 21 handler
		mov	    ds,ax		    ;		.
		lea	    dx,Catch21		    ;		.
		mov	    al,MsDos		    ;		.
		mov	    ah,DosSetVector	    ;		.
		int	    MsDos		    ;		.
		pop	    di			    ;Restore regs (MSC conv)
		pop	    si			    ;		.
		pop	    ds			    ;		.
		ret
_Grab21 	endp


;   Restore the original INT 21 vector
;
_GiveUp21	proc
		push	    ds			    ;Save regs (MSC conv)
		push	    si			    ;		.
		push	    di			    ;		.
		lds	    dx,CS:OldInt21	    ;Restore the old INT 21
		mov	    al,MsDos		    ;	 vector address
		mov	    ah,DosSetVector	    ;		.
		int	    MsDos		    ;		.
		pop	    di			    ;Restore regs (MSC conv)
		pop	    si			    ;		.
		pop	    ds			    ;		.
		ret
_GiveUp21	endp


;   INT 21 pre-processor code to redirect writes to STDOUT/STDERR
;
Catch21 	proc	    far
		cmp	    ah,DosWriteFile	    ;Is it a write function?
		jne	    Catch21_C		    ;  No: continue
		cmp	    bx,STDOUT		    ;Writing to STDOUT?
		je	    Catch21_A		    ;  Yes: redirect it
		cmp	    bx,STDERR		    ;Writing to STDERR?
		jne	    Catch21_C		    ;  No: continue
Catch21_A:	push	    ax			    ;Save all callers regs
		push	    bx			    ;		.
		push	    cx			    ;		.
		push	    dx			    ;		.
		push	    si			    ;		.
		push	    di			    ;		.
		push	    ds			    ;		.
		push	    es			    ;		.
		mov	    si,dx		    ;Count of char to write
Catch21_B:	mov	    al,[si]		    ;Write char to screen
		mov	    ah,BiosWriteTTY	    ;		.
		int	    BiosVideo		    ;		.
		inc	    si			    ;		.
		loop	    Catch21_B		    ;		.
		pop	    es			    ;Restore callers regs
		pop	    ds			    ;		.
		pop	    di			    ;		.
		pop	    si			    ;		.
		pop	    dx			    ;		.
		pop	    cx			    ;		.
		pop	    bx			    ;		.
		pop	    ax			    ;		.
Catch21_C:	jmp	    CS:OldInt21 	    ;Let INT 21 handle write
						    ;	 to file (redir)
Catch21 	endp

		end
