/***************************************************************************/
/*                                                                         */
/*  ROUTEMSG.C - redirect a program's STDOUT and STDERR to both the        */
/*       screen and a file.                                                */
/*                                                                         */
/*========================== Modification Log =============================*/
/*                                                                         */
/*  V01.00  07/01/89    Bob Withers Program initially complete.            */
/*                                                                         */
/***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <process.h>
#include <io.h>
#include <dos.h>

#define STDOUT                1
#define STDERR                2

#define VERSION               "V01.00"

extern void                   Grab21(void);
extern void                   GiveUp21(void);

static void                   ErrorMsg(char *pszMsg, ...);
static void                   Usage(void);

static char                  *pszOpenMode = "w";
static char                   wrk[64];

int main(int argc, char **argv)
{
    auto     FILE           *pFile;
    auto     int             hFile;
    auto     int             nRC = -1;

    if (argc > 1)
    {
        if ('-' == argv[1][0] || '/' == argv[1][0])
        {
            if ('A' == toupper(argv[1][1]))
            {
                pszOpenMode = "a";
                --argc;
                ++argv;
            }
            else
                Usage();
        }
    }

    if (argc < 3)
        Usage();

    if (NULL == (pFile = fopen(argv[1], pszOpenMode)))
        ErrorMsg("Unable to open file %s", argv[1]);

    hFile = fileno(pFile);
    dup2(hFile, STDOUT);
    dup2(hFile, STDERR);
    fclose(pFile);

    argc -= 2;
    memmove(argv, argv + 2, argc * sizeof(argv[0]));
    argv[argc] = NULL;

    Grab21();

    if (NULL == strchr(argv[0], '.'))
    {
        strcat(strcpy(wrk, argv[0]), ".COM");
        nRC = spawnvp(P_WAIT, wrk, argv);
    }

    if (-1 == nRC)
        nRC = spawnvp(P_WAIT, argv[0], argv);

    GiveUp21();

    return(nRC);
}


static void ErrorMsg(char *pszMsg, ...)
{
    auto     va_list        ap;

    printf("RouteMsg - ");
    va_start(ap, pszMsg);
    vprintf(pszMsg, ap);
    va_end(ap);
    exit(1);
}


static void Usage(void)
{
    printf("\nRouteMsg - Redirect STDOUT/STDERR for child process - %s\n"
           "           Bob Withers, 7/1/89\n\n"
           "     Usage:  routemsg [/a] <file> <cmd> [<args>]\n\n"
           "             /a     ==> Append program output to <file>\n"
           "             <file> ==> Output file for data capture\n"
           "             <cmd>  ==> Program to be executed\n"
           "             <args> ==> Command line arguments for <cmd>\n",
           VERSION);

    exit(2);
}
